import socket
import sys
import termios
import tty
import select

# socat alternative: socat STDIO,raw,echo=0,opost=1,onlcr=1,escape=0x1d TCP:10.0.4.20:23,crnl

ESCAPE_BYTE = b'\x1d'  # Ctrl+] to exit

def run_session(host: str, port: int, tcp_timeout: int) -> None:
    # Save original tty settings
    fd = sys.stdin.fileno()
    orig_attrs = termios.tcgetattr(fd)

    try:
        # Make stdin "raw" but keep output processing (opost/onlcr)
        attrs = termios.tcgetattr(fd)
        # lflag: disable canonical mode, echo and signals
        lflag = attrs[3]
        lflag &= ~termios.ICANON
        lflag &= ~termios.ECHO
        lflag &= ~termios.ISIG
        attrs[3] = lflag
        termios.tcsetattr(fd, termios.TCSADRAIN, attrs)

        # Connect TCP
        sock = socket.create_connection((host, port), timeout=tcp_timeout)

        # Main loop
        while True:
            rlist, _, _ = select.select([sys.stdin, sock], [], [])

            # Data from stdin
            if sys.stdin in rlist:
                data = sys.stdin.buffer.read(1)
                if not data:
                    # EOF on stdin
                    break

                if data == ESCAPE_BYTE:
                    # Local escape: close connection and exit
                    break

                # Translate '\n' to '\r\n' like crnl
                if data == b'\n':
                    sock.sendall(b'\r\n')
                else:
                    sock.sendall(data)

            # Data from socket
            if sock in rlist:
                data = sock.recv(4096)
                if not data:
                    # Connection closed
                    break
                # Just write to stdout as-is
                sys.stdout.buffer.write(data)
                sys.stdout.buffer.flush()

        sock.close()

    finally:
        # Restore terminal attributes
        termios.tcsetattr(fd, termios.TCSADRAIN, orig_attrs)


# if __name__ == "__main__":
#     run_session("10.0.4.20", 23)



import socket, select
import time
import logging
import re

logger = logging.getLogger(__name__)

def strip_ansi(data):
    _ANSI_RE = re.compile(
        r'(?:\x1B[@-Z\\-_]|\x1B\[[0-?]*[ -/]*[@-~]|\x1B\][^\x07\x1B]*(?:\x07|\x1B\\)|\x1B[P^_].*?\x1B\\)',
        re.DOTALL
    )
    clean = lambda s: _ANSI_RE.sub('', s)
    if isinstance(data, str):
        return clean(data)
    if isinstance(data, list):
        return [clean(s) for s in data]
    if isinstance(data, tuple):
        return tuple(clean(s) for s in data)
    # return data
    return [clean(s) for s in data]

def lines_clean(lines):
    lines = strip_ansi(lines)
    lines = [line for line in lines if line.strip()]    # remove empty
    return lines

def myrepr(obj):
    return repr(bytes(obj))

def parse_kv(data, separator=":"):
    res = {}
    for line in data:
        if line.find(":") != -1:
            kv = line.split(separator, 1)
            k = kv[0].strip()
            v = kv[1].strip()
            res[k] = v
    return res

class TCP_Stream:
    def __init__(self, portname: tuple[str, int]):
        self.address = portname[0]
        self.port = portname[1]
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transmitted = 0
        self.received = 0
        self.is_connected = False

    def open(self):
        logger.debug(f"Connect to {self.address}:{self.port}")
        self.socket.connect((self.address, self.port))
        if self.socket.fileno() != -1:
            self.is_connected = True

    def __del__(self):
        self.close()

    def close(self):
        logger.debug(f"Disconnect from {self.address}:{self.port}")
        self.socket.close()
        self.is_connected = False

    def write(self, data: bytes):
        self.transmitted += len(data)
        logger.debug(f" <  {myrepr(data)}")
        self.socket.sendall(data)

    def try_read(self, size: int):
        data = self.socket.recv(size)
        self.received += len(data)
        return data
    
    def try_read_timeout(self, size: int):
        readable, _, _ = select.select([self.socket], [], [], 0)
        if readable:
            size = 1
            data = self.socket.recv(size)
            self.received += len(data)
            # print(f"Received: {data}")
            return data
        return None

    def read(self, size: int):
        data = self.try_read(size)
        while len(data) < size:
            data += self.try_read(size - len(data))
        return data
    
    def read_timeout(self, size: int, timeout: float = 0.1):
        timeout = 0
        data = self.try_read_timeout(size)
        if data == None:
            data = b""
        ts = time.time()
        while len(data) < size and time.time() - ts < timeout:
            buf = self.try_read_timeout(size - len(data))
            if buf:
                data += buf
        return data
    
    def reset_input_buffer(self):
        pass
    
    @property
    def in_waiting(self):
        return 1

class BufferedRead:
    def __init__(self, stream: TCP_Stream):
        self.buffer = bytearray()
        self.stream = stream
    
    def until_timeout(self, eol: str = "\n", cut_eol: bool = True, timeout = None, timeout_assert = False):
        eol_bytes = eol.encode("ascii")
        ts = time.time()
        while time.time() - ts < timeout if timeout is not None else True:
            i = self.buffer.find(eol_bytes)
            if i >= 0:
                read = self.buffer[: i + len(eol_bytes)]
                logger.debug(f"  > {myrepr(read)}")
                if cut_eol:
                    read = self.buffer[:i]
                self.buffer = self.buffer[i + len(eol_bytes) :]
                return bytes(read)

            i = max(1, self.stream.in_waiting)
            data = self.stream.read_timeout(i, timeout=timeout)
            if data:
                self.buffer.extend(data)

        # Timeout reached, returning whatever is in the buffer
        ret = self.buffer
        logger.debug(f" W> {myrepr(ret)}")
        logger.warning(f"Timeout reached while waiting for \"{eol}\"")
        self.buffer = bytearray()
        
        # Assert
        if timeout is not None and timeout_assert:
            raise TimeoutError(f"Timeout reached while waiting for \"{eol}\". Check log file for details.")
        return bytes(ret)

class BSB_Lite():
    CLI_PROMPT_DEFAULT = ">: "
    CLI_EOL_DEFAULT = "\r\n"

    def __init__(self, portname: tuple[str, int], chunk_size: int = 1024):
        self.port = TCP_Stream(portname)
        self.read = BufferedRead(self.port)
        self.chunk_size = chunk_size
        self.CLI_PROMPT = self.CLI_PROMPT_DEFAULT
        self.CLI_EOL = self.CLI_EOL_DEFAULT

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stop()

    def start(self):
        self.port.open()
        self.port.reset_input_buffer()
        # Send a command with a known syntax to make sure the buffer is flushed
        self.send("\r")
        self.send("uptime\r")
        self.read.until_timeout("Uptime: ", timeout=None)
        # And read buffer until we get prompt
        self.read.until_timeout(self.CLI_PROMPT, timeout=None)

    def stop(self) -> None:
        self.port.close()

    def send(self, line: str) -> None:
        self.port.write(line.encode("ascii"))

    def read_until(self, eol: str = "\n", timeout: float = 0.1):
        return self.read.until_timeout(eol = eol, timeout=timeout)
    
    def read_until_eol(self, timeout = None):
        return self.read_until(self.CLI_EOL, timeout=timeout)

    def read_until_prompt(self, timeout = None):
        return self.read_until(self.CLI_PROMPT, timeout=timeout)

    def status_lights(self, r, g, b):
        assert 0 <= r <= 255
        assert 0 <= g <= 255
        assert 0 <= b <= 255
        self.send(f"status_lights {r} {g} {b}\r")
        return self.read_until_prompt(timeout=None).decode("ascii").split(self.CLI_EOL)

    def device_info(self, timeout=3*2):
        self.send("device_info\r")
        lines = self.read_until_prompt(timeout=timeout).decode("ascii").split(self.CLI_EOL)
        return parse_kv(lines)

    def cmd_oneshot(self, cmd: str, timeout=1):
        self.send(f"{cmd}\r")
        lines = self.read_until_prompt(timeout=timeout).decode("ascii").split(self.CLI_EOL)
        lines = [line for line in lines if line.strip() != cmd.strip()] # remove command itself
        lines = lines_clean(lines)
        return lines

    def sysctl_debug(self, value: bool):
        value = int(value)
        cmd = f"sysctl debug {value}\r"
        return self.cmd_oneshot(cmd, timeout=1)

    # def cmd_sl_cli_enter(self, timeout=3):
    #     self.CLI_PROMPT = "917>: "
    #     return self.cmd_oneshot("sl_cli", timeout=timeout)

    # def cmd_sl_cli_exit(self, timeout=1):
    #     self.CLI_PROMPT = self.CLI_PROMPT_DEFAULT
    #     return self.cmd_oneshot("exit", timeout=timeout)

    # def cmd_sl_cli_wifi_rf_test_enter(self, timeout=5):
    #     self.CLI_PROMPT = "wifi_rf_test>: "
    #     return self.cmd_oneshot("wifi_rf_test", timeout=timeout)

    # def cmd_sl_cli_wifi_rf_test_exit(self, timeout=3):
    #     self.CLI_PROMPT = "917>: "
    #     return self.cmd_oneshot("exit", timeout=timeout)

    # def cmd_sl_cli_ble_per_test_enter(self, timeout=5):
    #     self.CLI_PROMPT = "ble_per>: "
    #     return self.cmd_oneshot("ble_per_test", timeout=timeout)

    # def cmd_sl_cli_ble_per_test_exit(self, timeout=3):
    #     self.CLI_PROMPT = "917>: "
    #     return self.cmd_oneshot("exit", timeout=timeout)



import logging
import os, sys
import json

import http.client
from urllib.parse import urlparse
from urllib import request

import shutil, platform
import re, hashlib

from busybar_tools.config import PROJECT_NAME, FETCH_TIMEOUT_DEFAULT, UPDATE_SERVER_BASE
    
def setup_logging():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s')

    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    # fh = logging.FileHandler("tmp/app.log")
    # fh.setLevel(logging.DEBUG)
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)

def print_pretty(data, return_instead_of_print=False):
    def _out_handler(data):
        if return_instead_of_print:
            return data
        else:
            print(data)

    def _convert_to_json(data):
        if isinstance(data, dict) or isinstance(data, list):
            return json.dumps(data, indent=4)
        try:
            parsed_json = json.loads(data)
            return json.dumps(parsed_json, indent=4)
        except (json.JSONDecodeError, TypeError):
            return data

    if isinstance(data, str):
        try:
            parsed_data = json.loads(data)
            return _out_handler(_convert_to_json(parsed_data))
        except (json.JSONDecodeError, TypeError):
            return _out_handler(data)
    else:
        return _out_handler(_convert_to_json(data))

def file_download(file_url, file_name, dir):
    logging.info(f"Downloading {file_url} to {dir} ...")
    os.makedirs(dir, exist_ok=True)

    file_path = os.path.join(dir, file_name)

    request.urlretrieve(file_url, file_path)
    if os.path.isfile(file_path):
        return file_path
    return False


def busybar_workdir_get(subdir = None, suffix_len=4) -> str:
    script_path = os.path.dirname(os.path.abspath(__file__))
    script_hash = hashlib.sha256(script_path.encode()).hexdigest()[:suffix_len]

    # cross-platform way to get a persistent tmp dir
    TMP_SUBDIR = f"{PROJECT_NAME}_{script_hash}"
    if platform.system() == "Windows":
        tmp_dir = os.path.join(os.getenv("TEMP"), TMP_SUBDIR)
    else:
        tmp_dir = os.path.join("/tmp", TMP_SUBDIR)

    if subdir is not None:
        tmp_dir = os.path.join(tmp_dir, subdir)

    os.makedirs(tmp_dir, exist_ok=True)

    return tmp_dir

def url_to_dir_name(url: str) -> str:
    base = re.sub(r'[^a-z0-9_]+', '_', url.lower())
    base = re.sub(r'_+', '_', base).strip('_') or '_'

    # Cleaning:
    base = base.replace('https_', '')
    base = base.replace('http_', '')
    base = base.replace('update_flipperzero_one_builds_', '')

    base = base[:64]    # Crop to 64 chars

    h = hashlib.sha256(url.encode('utf-8')).hexdigest()[:8]
    return f"{base}_{h}"


def fetch_url(url, timeout=FETCH_TIMEOUT_DEFAULT):
    try:
        with request.urlopen(url, timeout=timeout) as response:
            if response.status == 200:
                return response.read().decode()
    except Exception as e:
        logging.error(f"Error fetching {url}: {e}")
    return None


# def busybar_api_update(device_ip, upd_bundle_tar, verbose=True):
#     logging.info(f"Uploading {upd_bundle_tar} to {device_ip}")
#     url = f"http://{device_ip}/api/update"
#     parsed = urlparse(url)

#     conn = http.client.HTTPConnection(parsed.hostname, parsed.port or 80)

#     with open(upd_bundle_tar, "rb") as f:
#         data = f.read()

#     headers = {
#         "Content-Type": "application/octet-stream",
#         "Content-Length": str(len(data)),
#     }

#     conn.request("POST", parsed.path, body=data, headers=headers)
#     response = conn.getresponse()

#     if verbose:
#         print("Status:", response.status, response.reason)
#         print("Headers:")
#         for k, v in response.getheaders():
#             print(f"  {k}: {v}")
#         print("Body:")
        
#         print(response.read().decode(errors="replace"))

#     conn.close()

#     return response.status


def busybar_api_update(device_ip, upd_bundle_tar, verbose=True):
    logging.info(f"Uploading {upd_bundle_tar} to {device_ip}")
    url = f"http://{device_ip}/api/update"
    parsed = urlparse(url)

    total_size = os.path.getsize(upd_bundle_tar)

    conn = http.client.HTTPConnection(parsed.hostname, parsed.port or 80)
    try:
        conn.putrequest("POST", parsed.path or "/")
        conn.putheader("Content-Type", "application/octet-stream")
        conn.putheader("Content-Length", str(total_size))
        conn.endheaders()

        sent = 0
        chunk_size = 65536
        last_percent = -1

        with open(upd_bundle_tar, "rb") as f:
            while True:
                chunk = f.read(chunk_size)
                if not chunk:
                    break
                conn.send(chunk)
                sent += len(chunk)

                percent = int(sent * 100 / total_size) if total_size else 100
                if percent != last_percent:
                    if verbose:
                        print(f"\rUpload progress: {percent:3d}%", end="", file=sys.stdout, flush=True)
                    last_percent = percent

        if last_percent >= 0:
            if verbose:
                print(file=sys.stdout, flush=True)

        response = conn.getresponse()

        if verbose:
            print("Status:", response.status, response.reason)
            print("Headers:")
            for k, v in response.getheaders():
                print(f"  {k}: {v}")
            print("Body:")
            body = response.read().decode(errors="replace")
            try:
                print_pretty(json.loads(body))
            except json.JSONDecodeError:
                print(body)
        else:
            _ = response.read()

        ret = response.status
        if ret == 200:
            return 0

    finally:
        conn.close()
    
    return 1


def busybar_type_by_filename(filename: str) -> None:
    # example:
    # busybar-f21-sil_firmware-porta_furi-state-02102025-473f0eb6.elf
    # ('f21', 'sil_firmware', 'factory-02102025-01b3a988', None, 'factory-02102025-01b3a988', 'elf')
    # https://github.com/flipperdevices/flipper-update-indexer/blob/cba82ded47e9b3aeb278a7b1925d0fc1faa4113a/indexer/src/models.py#L237
    # regex = re.compile(
    #     r"^busybar-(\w+)-(\w+)-([0-9.]+(-rc)?|(dev-\w+-\w+))\.(\w+)$"
    # )
    # regex = re.compile(
    #     r"^busybar-(\w+)-(\w+)-([0-9.]+(-rc)?|(\w+-\w+-\w+))\.(\w+)$"
    # )
    regex = re.compile(
        r"^busybar-(\w+)-(\w+)-(.*)\.(\w+)$"
    )
    match = regex.match(filename)
    if not match:
        exception_msg = f"Unknown file {filename}"
        logging.exception(exception_msg)
        raise Exception(exception_msg)
    # print_pretty(match.groups())
    type = match.group(2) + "_" + match.group(4)
    return type

def busybar_update_get_index_file_name(target_num):
    return f"busybar-f{int(target_num)}-sha256sum.txt"

def busybar_update_parse_index(index_data, url_base):
    files = []
    lines = index_data.splitlines()
    for line in lines:
        hash, filename = line.split(maxsplit=1)
        hash = hash.strip()
        filename = os.path.basename(filename.strip())

        file = {
            "sha256sum": hash,
            "file_name": filename,
            "file_url": url_base + filename,
            "file_type": busybar_type_by_filename(filename)
        }
        files.append(file)

    return files

def busybar_update_url_normalize(branch):
    if branch.startswith("https://"):
        url = branch
    elif branch.startswith("http://"):
        url = branch
    else:
        if branch.startswith("/"):
            branch = branch[1:]

        if UPDATE_SERVER_BASE.endswith('/'):
            url = UPDATE_SERVER_BASE + branch
        else:
            url = UPDATE_SERVER_BASE + '/' + branch

    if not url.endswith('/'):
        url += '/'

    return url

def file_sha256(file_path):
    try:
        sha256 = hashlib.sha256()
        with open(file_path, "rb") as f:
            while chunk := f.read(65536):
                sha256.update(chunk)
        return sha256.hexdigest()
    except Exception as e:
        logging.warning(f"Can't calculate SHA256 for {file_path}: {e}")
    return None

def bundle_download(files, dir):
    os.makedirs(dir, exist_ok=True)
    
    for file in files:
        if file_download(file, dir):
            print("OK")
        else:
            print(f"Failed!")





PROJECT_NAME = "busybar_tools"

# Network:
UPDATE_SERVER_BASE = "https://update.flipperzero.one/builds/busybar-firmware/"
UPDATE_DEFAULT_BRANCH = "dev"

FETCH_TIMEOUT_DEFAULT = 60  # seconds
TCP_TIMEOUT_DEFAULT = 5  # seconds

# Device:
DEVICE_IP = "10.0.4.20"
DEVICE_IP_REF = "10.0.5.20" # misc
DEVICE_PORT = 23

# Firmware U5 target:
U5_TARGET_HW = 21   # Default, can be overridden by -t / --target option.
U5_TARGET_HW_OPTIONS = [20, 21, 22]



import os
import sys
import argparse
import logging

from busybar_tools import (
    run_update_via_http,
    run_clean,
    run_cli_terminal
)

from busybar_tools.helpers import (
    setup_logging
)

from busybar_tools.config import (
    DEVICE_IP,
    DEVICE_IP_REF,
    DEVICE_PORT,
    U5_TARGET_HW,
    U5_TARGET_HW_OPTIONS,
    UPDATE_DEFAULT_BRANCH
)

def busybar_main():
    logging.debug(f"cwd: {os.getcwd()}")

    parser = argparse.ArgumentParser(description="Runner")
    # parser.add_argument("-v", "--verbose", help="Verbose", action="store_true")
    
    parser.add_argument("-d", "--device", help="Device IP", type=str, default=DEVICE_IP, action="store")
    parser.add_argument("-p", "--port", help="Device Port", type=int, default=DEVICE_PORT, action="store")
    parser.add_argument("-t", "--target", help="Target hardware", type=int, default=U5_TARGET_HW, action="store", choices=U5_TARGET_HW_OPTIONS)
    parser.add_argument("-n", "--no-action", help="No action, dry run", action="store_true")

    parser.parse_known_args()

    subparsers = parser.add_subparsers(
        dest="command", help="Commands to run", required=False
    )

    p_run_update_http = subparsers.add_parser(
        "update", help="Update firmware via HTTP API"
    )
    p_run_update_http.add_argument("branch", help="Branch to update", type=str, default=UPDATE_DEFAULT_BRANCH, nargs='?')
    p_run_update_http.set_defaults(func=run_update_via_http)

    p_run_cli = subparsers.add_parser(
        "cli", help="CLI terminal session to device"
    )
    p_run_cli.set_defaults(func=run_cli_terminal)

    p_clean = subparsers.add_parser(
        "clean", help="Clean package's tmp directory"
    )
    p_clean.set_defaults(func=run_clean)

    # p_flash_u5_dfu = subparsers.add_parser(
    #     "flash-u5-dfu", help="Flash U5 firmware via DFU"
    # )
    # p_flash_u5_dfu.add_argument("-d", "--device_ip", help="Device IP", type=str, default=DEVICE_IP)
    # p_flash_u5_dfu.add_argument("-p", "--device_port", help="Device Port", type=int, default=DEVICE_PORT)
    # p_flash_u5_dfu.set_defaults(func=run_flash_u5_dfu)

    # p_flash_si_uart = subparsers.add_parser(
    #     "flash-si-uart", help="Flash SI917 firmware via UART"
    # )
    # p_flash_si_uart.add_argument("-s", "--serial_port", help="Serial port for SI917", type=str, default=None, required=False)
    # p_flash_si_uart.add_argument("-f", "--firmware_path", help="Path to the SI917 firmware .rps file", type=str, default=None, required=False)
    # p_flash_si_uart.set_defaults(func=run_flash_si_uart)

    # p_flash_si_nwp_uart = subparsers.add_parser(
    #     "flash-si-nwp-uart", help="Flash SI917 NWP firmware via UART"
    # )
    # p_flash_si_nwp_uart.add_argument("-s", "--serial_port", help="Serial port for SI917", type=str, default=None, required=False)
    # p_flash_si_nwp_uart.add_argument(
    #     "-f", "--firmware_path", help="Path to the SI917 NWP firmware .rps file", type=str, default=None, required=False
    # )
    # p_flash_si_nwp_uart.set_defaults(func=run_flash_si_nwp_uart)

    # p_update_via_http = subparsers.add_parser(
    #     "update-http", help="Update device via HTTP API using curl (upd_bundle.tar)"
    # )
    # p_update_via_http.add_argument("-d", "--device_ip", help="Device IP", type=str, default=DEVICE_IP)
    # p_update_via_http.add_argument("-p", "--device_port", help="Device Port", type=int, default=DEVICE_PORT)
    # p_update_via_http.set_defaults(func=run_update_via_http)

    # p_update_via_storage = subparsers.add_parser(
    #     "update-storage", help="Update device via storage.py (update bundle)"
    # )
    # p_update_via_storage.add_argument("-d", "--device_ip", help="Device IP", type=str, default=DEVICE_IP)
    # p_update_via_storage.add_argument("-p", "--device_port", help="Device Port", type=int, default=DEVICE_PORT)
    # p_update_via_storage.set_defaults(func=run_update_via_storage)

    # p_wait_for_device = subparsers.add_parser(
    #     "wait", help="Just wait for device to be reachable via ping, nothing else"
    # )
    # p_wait_for_device.add_argument("-d", "--device_ip", help="Device IP", type=str, default=DEVICE_IP)
    # p_wait_for_device.set_defaults(func=run_wait_for_device)
    

    args = parser.parse_args()

    if args.device in ["r", "R", "ref"]:
        args.device = DEVICE_IP_REF

    args.verbose = True

    if args.command is not None:
        return args.func(args)
    else:
        parser.print_help()


def main():
    setup_logging()

    try:
        ret = busybar_main()
        print("RET: ", ret)
        if ret and ret != 0:
            print("Run: Exiting with error code", ret, file=sys.stderr)
            sys.exit(1)
    except KeyboardInterrupt:
        print("Run: Exited", file=sys.stderr)
        sys.exit(2)
    # except subprocess.CalledProcessError as e:
    #     sys.exit(e.returncode)
    except Exception as e:
        print(f"Run: Error: {e}", file=sys.stderr)
        sys.exit(3)


# if __name__ == "__main__":
#     main()



#!/usr/bin/env python3
import hashlib
import os, sys, time
import subprocess, argparse
import shutil, platform
import json


import json
import os
import re
from urllib import request
import logging

import http.client
import os
from urllib.parse import urlparse

from busybar_tools.helpers import fetch_url, print_pretty, file_download, busybar_workdir_get, url_to_dir_name, busybar_api_update, busybar_update_get_index_file_name, busybar_update_url_normalize, busybar_update_parse_index, file_sha256

from busybar_tools.bsb_term import run_session

from busybar_tools.bsb_lite import BSB_Lite

from busybar_tools.config import TCP_TIMEOUT_DEFAULT


def bsb_debug_enable(args):
    logging.info("Try to enable debug mode...")

    bsb = BSB_Lite((args.device, args.port))
    bsb.start()
    res = bsb.sysctl_debug(1)
    print_pretty(res)

def run_cli_terminal(args):
    logging.info("Running CLI terminal...")
    # print_pretty(args)

    timeout = TCP_TIMEOUT_DEFAULT

    print(f"Connecting to {args.device}:{args.port} with timeout {timeout}s...")
    print("Press Ctrl+] to exit.")

    run_session(args.device, args.port, tcp_timeout=timeout)

def run_update_via_http(args):
    logging.info("Running update via HTTP...")
    # print_pretty(args)

    base_url = busybar_update_url_normalize(args.branch)
    logging.info(f"URL: {base_url}")

    work_dir = busybar_workdir_get(url_to_dir_name(base_url))
    logging.info(f"Workdir: {work_dir}")

    index_name = busybar_update_get_index_file_name(args.target)
    index_url = f"{base_url}{index_name}"

    try:
        index_data = fetch_url(index_url, timeout=10)
        if not index_data:
            raise Exception(f"Failed to fetch index {index_url}")
    except Exception as e:
        index_data = None
        logging.warning(f"{e}")
        logging.info("Trying to use cached index...")

    index_path = os.path.join(work_dir, index_name)
    if index_data:
        try:
            with open(index_path, 'w') as f:
                f.write(index_data)
            logging.info(f"Index content saved to {index_path}")
        except Exception as e:
            logging.error(f"Error saving index content: {e}")
    else:
        try:
            with open(index_path, 'r') as f:
                index_data = f.read()
            logging.info(f"Loaded index content from cache {index_path}")
        except Exception as e:
            logging.error(f"Error loading index content from cache: {e}")
            return 1

    index_parsed = busybar_update_parse_index(index_data, base_url)
    # print_pretty(index_parsed)

    file_path = None
    for file in index_parsed:
        if file["file_type"] == "update_tar":
            file_path = os.path.join(work_dir, file['file_name'])
            if file_sha256(file_path) != file["sha256sum"]:
                logging.warning(f"File missing or Hash check failed: {file['file_name']}")

                file_path = file_download(file["file_url"], file['file_name'], work_dir)
            
            if file_sha256(file_path) == file["sha256sum"]:
                logging.info(f"Hash check passed: {file['file_name']}: {file['sha256sum']}")
            else:
                logging.error(f"Hash check failed ONCE AGAIN: {file['file_name']}: {file['sha256sum']}")
            break

    if file_path:
        if args.no_action:
            print("No action specified, not uploading.")
            return 0
        bsb_debug_enable(args)
        return busybar_api_update(args.device, file_path)
    else:
        logging.error("No update file found.")
        return 1

def run_clean(args):
    dir = busybar_workdir_get()
    print(f"Cleaning up {dir}...")

    try:
        shutil.rmtree(dir, ignore_errors=True)
    except Exception as e:
        logging.error(f"Error cleaning up {dir}: {e}")
        return 1

    return 0

